clear,clc

addpath('./KPCA');
addpath('./Superpixel');
addpath('./GaborFunction');
addpath('./RF_Class_C');

load('./Dataset/Salinas.mat');
hsi_data_original = double(Salinas_corrected);
AllTrueLabel = double(Salinas_gt);
[X_hsi, Y_hsi, B_hsi] =size(hsi_data_original);
Res = 3.7;

[trainall, train_sec_idx] = create_sample(AllTrueLabel);
[m,n] = size(train_sec_idx);

Labels = unique(AllTrueLabel);
Labels = Labels(2:end);
AllLabel = double(reshape(AllTrueLabel,1,size(AllTrueLabel,1)*size(AllTrueLabel,2)));
num_classes = size(unique(AllTrueLabel),1)-1;
label_num = size(AllLabel,2);

%%%%%%%%%%%%%%  PCA  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[hsi_input_data, ~] = PCA_img(hsi_data_original,'first');

%%%%%%%%%%%%%% Superpixel %%%%%%%%%%%%%%%%%%%%%%%%
I = 7;
distArr = CreatSuperpixel(hsi_input_data, Res, I);

%%%%%%%%%%%%%% HSI Gabor Feature Extraction %%%%%%%%%%%%%%%%%%%%%%%%
parameters_Ks = {[1,1,1,1], [1,1,2,1], [1,1,3,2], [2,2,1,2], [2,2,2,2]};
kers_max = 53;
f_pro = CreateFeature(hsi_data_original, AllTrueLabel, parameters_Ks, kers_max, distArr, trainall, train_sec_idx );


%%%%%%%%%%%%%% Classification %%%%%%%%%%%%%%%%%%%%%%%%
pre_labels=cell(m,n);
pre_labels_test = cell(m,n);
rate=zeros(m,n);
kappa_acc = zeros(m,n);
average_acc = zeros(m,n);
overall_acc = zeros(m,n);
class_acc = cell(m,n);

for train_ite =1:13
    
    for train_no = 1:n
        fpro = zeros(X_hsi,Y_hsi,num_classes);
        for col = 1:size(f_pro,2)
            fpro = fpro + f_pro{col}{train_ite,train_no};
        end
        fpro = fpro ./ size(f_pro,2);
        train_index = train_sec_idx{train_ite,train_no}';
        train_label = trainall(2, train_index);
        
        test_index = 1:size(trainall,2);
        test_label = trainall(2,test_index);
        
        [value, index] = max(fpro,[],3);       
        pre_label = reshape(index, [1,X_hsi*Y_hsi]);
        pre_labels{train_ite,train_no} = pre_label;
        pre_label_test = pre_label(trainall(1,:));
        pre_labels_test{train_ite,train_no} = pre_label_test;
        rate(train_ite,train_no)=mean(pre_label_test == test_label);
        
        [overall_accuracy,kappa_accuracy,average_accuracy,class_accuracy,total_corrected_num,errorMatrix] = calcError(test_label, pre_label_test, Labels);
        overall_acc(train_ite,train_no) = overall_accuracy;
        kappa_acc(train_ite,train_no) = kappa_accuracy;
        average_acc(train_ite,train_no) = average_accuracy;
        class_acc{train_ite,train_no} = class_accuracy;
       
    end
end
for train_ite = 1:m
    rate(train_ite,n+1) = mean(rate(train_ite,1:n));
    overall_acc(train_ite,n+1) =mean( overall_acc(train_ite,1:n));
    kappa_acc(train_ite,n+1) =mean( kappa_acc(train_ite,1:n));
end

SuVGF.pre_labels = pre_labels;
SuVGF.pre_label_test  = pre_labels_test ;
SuVGF.overall_acc = overall_acc;
SuVGF.kappa_acc = kappa_acc;
SuVGF.average_acc = average_acc;
SuVGF.class_acc = class_acc;
SuVGF.pro = pro_acc;
SuVGF.rate = rate;

file =  ['./SuVGF_Salinas.mat'];
save(file,'SuVGF','-v7.3');